>From f49966c9be8d1f7d58b14b2394264b412127e31e Mon Sep 17 00:00:00 2001
From: Takashi Iwai <tiwai@suse.de>
Date: Wed, 24 Mar 2021 09:13:29 +0100
Subject: [PATCH 6/7] ALSA: hda: Add frozen flag for prohibiting hardware
 access

The PCI rescan feature we're going to implement requires the prohibit
of the hardware access during rescan and remap.  This patch introduce
a flag, frozen, to hdac_bus for preventing the CORB/RIRB accesses, as
well as the helper function to flip it up/down.

Signed-off-by: Takashi Iwai <tiwai@suse.de>
---
 include/sound/hdaudio.h |  2 ++
 sound/hda/hdac_bus.c    | 13 ++++++++++++-
 2 files changed, 14 insertions(+), 1 deletion(-)

diff --git a/include/sound/hdaudio.h b/include/sound/hdaudio.h
index 22af68b01426..b270dd1f63ac 100644
--- a/include/sound/hdaudio.h
+++ b/include/sound/hdaudio.h
@@ -333,6 +333,7 @@ struct hdac_bus {
 
 	/* operation state */
 	bool chip_init:1;		/* h/w initialized */
+	bool frozen:1;			/* h/w access prohibited */
 
 	/* behavior flags */
 	bool aligned_mmio:1;		/* aligned MMIO access */
@@ -377,6 +378,7 @@ struct hdac_bus {
 int snd_hdac_bus_init(struct hdac_bus *bus, struct device *dev,
 		      const struct hdac_bus_ops *ops);
 void snd_hdac_bus_exit(struct hdac_bus *bus);
+void snd_hdac_bus_freeze(struct hdac_bus *bus, bool freeze);
 int snd_hdac_bus_exec_verb_unlocked(struct hdac_bus *bus, unsigned int addr,
 				    unsigned int cmd, unsigned int *res);
 
diff --git a/sound/hda/hdac_bus.c b/sound/hda/hdac_bus.c
index 71db8592b33d..91e8f3c25a08 100644
--- a/sound/hda/hdac_bus.c
+++ b/sound/hda/hdac_bus.c
@@ -75,6 +75,14 @@ void snd_hdac_bus_exit(struct hdac_bus *bus)
 }
 EXPORT_SYMBOL_GPL(snd_hdac_bus_exit);
 
+void snd_hdac_bus_freeze(struct hdac_bus *bus, bool frozen)
+{
+	mutex_lock(&bus->cmd_mutex);
+	bus->frozen = frozen;
+	mutex_unlock(&bus->cmd_mutex);
+}
+EXPORT_SYMBOL_GPL(snd_hdac_bus_freeze);
+
 /**
  * snd_hdac_bus_exec_verb - execute a HD-audio verb on the given bus
  * @bus: bus object
@@ -90,7 +98,10 @@ int snd_hdac_bus_exec_verb(struct hdac_bus *bus, unsigned int addr,
 	int err;
 
 	mutex_lock(&bus->cmd_mutex);
-	err = snd_hdac_bus_exec_verb_unlocked(bus, addr, cmd, res);
+	if (bus->frozen)
+		err = -EBADFD;
+	else
+		err = snd_hdac_bus_exec_verb_unlocked(bus, addr, cmd, res);
 	mutex_unlock(&bus->cmd_mutex);
 	return err;
 }
-- 
2.26.2

